@@ -64,6 +64,8 @@ namespace Sass {
     virtual void operator()(String_Schema*);
     virtual void operator()(String_Constant*);
     virtual void operator()(String_Quoted*);
+    virtual void operator()(Custom_Error*);
+    virtual void operator()(Custom_Warning*);
     virtual void operator()(Supports_Operator*);
     virtual void operator()(Supports_Negation*);
     virtual void operator()(Supports_Declaration*);
