@@ -16,15 +16,13 @@ namespace Sass {
 
   class Listize : public Operation_CRTP<Expression*, Listize> {
 
-    Context&            ctx;
+    Memory_Manager& mem;
 
     Expression* fallback_impl(AST_Node* n);
 
   public:
-    Listize(Context&);
-    virtual ~Listize() { }
-
-    using Operation<Expression*>::operator();
+    Listize(Memory_Manager&);
+    ~Listize() { }
 
     Expression* operator()(Selector_List*);
     Expression* operator()(Complex_Selector*);
