@@ -1,3 +1,4 @@
+#include "sass.hpp"
 #include <cctype>
 #include <cstddef>
 #include <iostream>
@@ -91,6 +92,8 @@ namespace Sass {
                unicode,
                exactly<'-'>,
                exactly<'_'>,
+               NONASCII,
+               ESCAPE,
                escape_seq
              >(src);
     }
@@ -104,6 +107,8 @@ namespace Sass {
                unicode,
                exactly<'-'>,
                exactly<'_'>,
+               NONASCII,
+               ESCAPE,
                escape_seq
              >(src);
     }
@@ -149,8 +154,8 @@ namespace Sass {
              >(src);
     }
 
-    // Match CSS unit identifier.
-    const char* unit_identifier(const char* src)
+    // Match a single CSS unit
+    const char* one_unit(const char* src)
     {
       return sequence <
                optional < exactly <'-'> >,
@@ -165,6 +170,34 @@ namespace Sass {
              >(src);
     }
 
+    // Match numerator/denominator CSS units
+    const char* multiple_units(const char* src)
+    {
+      return
+        sequence <
+          one_unit,
+          zero_plus <
+            sequence <
+              exactly <'*'>,
+              one_unit
+            >
+          >
+        >(src);
+    }
+
+    // Match complex CSS unit identifiers
+    const char* unit_identifier(const char* src)
+    {
+      return sequence <
+        multiple_units,
+        optional <
+          sequence <
+          exactly <'/'>,
+          multiple_units
+        > >
+      >(src);
+    }
+
     const char* identifier_alnums(const char* src)
     {
       return one_plus< identifier_alnum >(src);
@@ -190,7 +223,12 @@ namespace Sass {
                  sequence <
                    zero_plus <
                      alternatives <
-                       identifier,
+                       sequence <
+                         optional <
+                           exactly <'$'>
+                         >,
+                         identifier
+                       >,
                        exactly <'-'>
                      >
                    >,
@@ -198,9 +236,13 @@ namespace Sass {
                    zero_plus <
                      alternatives <
                        digits,
-                       identifier,
+                       sequence <
+                         optional <
+                           exactly <'$'>
+                         >,
+                         identifier
+                       >,
                        quoted_string,
-                       exactly<'+'>,
                        exactly<'-'>
                      >
                    >
@@ -274,18 +316,54 @@ namespace Sass {
       >(src);
     }
 
-    const char* value_schema(const char* src) {
-      // follows this pattern: ([xyz]*i[xyz]*)+
-      return one_plus< sequence< zero_plus< alternatives< identifier, percentage, dimension, hex, number, quoted_string > >,
-                                 interpolant,
-                                 zero_plus< alternatives< identifier, percentage, dimension, hex, number, quoted_string, exactly<'%'> > > > >(src);
+    const char* sass_value(const char* src) {
+      return alternatives <
+        quoted_string,
+        identifier,
+        percentage,
+        hex,
+        dimension,
+        number
+      >(src);
     }
 
-    /* not used anymore - remove?
-    const char* filename(const char* src) {
-      return one_plus< alternatives< identifier, number, exactly<'.'> > >(src);
+    // this is basically `one_plus < sass_value >`
+    // takes care to not parse invalid combinations
+    const char* value_combinations(const char* src) {
+      // `2px-2px` is invalid combo
+      bool was_number = false;
+      const char* pos = src;
+      while (src) {
+        if ((pos = alternatives < quoted_string, identifier, percentage, hex >(src))) {
+          was_number = false;
+          src = pos;
+        } else if (!was_number && !exactly<'+'>(src) && (pos = alternatives < dimension, number >(src))) {
+          was_number = true;
+          src = pos;
+        } else {
+          break;
+        }
+      }
+      return src;
+    }
+
+    // must be at least one interpolant
+    // can be surrounded by sass values
+    // make sure to never parse (dim)(dim)
+    // since this wrongly consumes `2px-1px`
+    // `2px1px` is valid number (unit `px1px`)
+    const char* value_schema(const char* src)
+    {
+      return sequence <
+        one_plus <
+          sequence <
+            optional < value_combinations >,
+            interpolant,
+            optional < value_combinations >
+          >
+        >
+      >(src);
     }
-    */
 
     // Match CSS '@' keywords.
     const char* at_keyword(const char* src) {
@@ -394,7 +472,7 @@ namespace Sass {
     }
     const char* elseif_directive(const char* src) {
       return sequence< exactly< else_kwd >,
-                                optional_css_whitespace,
+                                optional_css_comments,
                                 word< if_after_else_kwd > >(src);
     }
 
@@ -535,6 +613,9 @@ namespace Sass {
     }
     // Match CSS numeric constants.
 
+    const char* op(const char* src) {
+      return class_char<op_chars>(src);
+    }
     const char* sign(const char* src) {
       return class_char<sign_chars>(src);
     }
@@ -599,23 +680,19 @@ namespace Sass {
     // Match CSS uri specifiers.
 
     const char* uri_prefix(const char* src) {
-      return exactly<url_kwd>(src);
-    }
-    const char* uri_value(const char* src)
-    {
-      return
-      sequence <
-        negate <
-          exactly < '$' >
+      return sequence <
+        exactly <
+          url_kwd
         >,
         zero_plus <
-          alternatives <
-            alnum,
-            interpolant,
-            exactly <'/'>,
-            class_char < uri_chars >
+          sequence <
+            exactly <'-'>,
+            one_plus <
+              alpha
+            >
           >
-        >
+        >,
+        exactly <'('>
       >(src);
     }
 
@@ -755,14 +832,17 @@ namespace Sass {
     const char* kwd_false(const char* src) {
       return word<false_kwd>(src);
     }
+    const char* kwd_only(const char* src) {
+      return keyword < only_kwd >(src);
+    }
     const char* kwd_and(const char* src) {
-      return word<and_kwd>(src);
+      return keyword < and_kwd >(src);
     }
     const char* kwd_or(const char* src) {
-      return word<or_kwd>(src);
+      return keyword < or_kwd >(src);
     }
     const char* kwd_not(const char* src) {
-      return word<not_kwd>(src);
+      return keyword < not_kwd >(src);
     }
     const char* kwd_eq(const char* src) {
       return exactly<eq>(src);
@@ -944,8 +1024,10 @@ namespace Sass {
         UUNICODE,
         sequence<
           exactly<'\\'>,
-          NONASCII,
-          class_char< escape_chars >
+          alternatives<
+            NONASCII,
+            escapable_character
+          >
         >
       >(src);
     }
@@ -1010,7 +1092,7 @@ namespace Sass {
                            hex,
                            exactly<'|'>,
                            // exactly<'+'>,
-                           sequence < number, identifier >,
+                           sequence < number, unit_identifier >,
                            number,
                            sequence< exactly<'!'>, word<important_kwd> >
                           >(src);
@@ -1098,34 +1180,39 @@ namespace Sass {
       return sequence< number, optional_spaces, exactly<'/'>, optional_spaces, number >(src);
     }
 
-    template <size_t size, prelexer mx, prelexer pad>
-    const char* padded_token(const char* src)
-    {
-      size_t got = 0;
-      const char* pos = src;
-      while (got < size) {
-        if (!mx(pos)) break;
-        ++ pos; ++ got;
-      }
-      while (got < size) {
-        if (!pad(pos)) break;
-        ++ pos; ++ got;
-      }
-      return got ? pos : 0;
-    }
-
-    template <size_t min, size_t max, prelexer mx>
-    const char* minmax_range(const char* src)
-    {
-      size_t got = 0;
-      const char* pos = src;
-      while (got < max) {
-        if (!mx(pos)) break;
-        ++ pos; ++ got;
-      }
-      if (got < min) return 0;
-      if (got > max) return 0;
-      return pos;
+    // lexer special_fn: these functions cannot be overloaded
+    // (/((-[\w-]+-)?(calc|element)|expression|progid:[a-z\.]*)\(/i)
+    const char* re_special_fun(const char* src) {
+      return sequence <
+        optional <
+          sequence <
+            exactly <'-'>,
+            one_plus <
+              alternatives <
+                alpha,
+                exactly <'+'>,
+                exactly <'-'>
+              >
+            >
+          >
+        >,
+        alternatives <
+          exactly < calc_fn_kwd >,
+          exactly < expression_kwd >,
+          sequence <
+            sequence <
+              exactly < progid_kwd >,
+              exactly <':'>
+            >,
+            zero_plus <
+              alternatives <
+                char_range <'a', 'z'>,
+                exactly <'.'>
+              >
+            >
+          >
+        >
+      >(src);
     }
 
   }
