@@ -21,7 +21,7 @@ namespace Sass {
     To_Value(Context& ctx, Memory_Manager& mem)
     : ctx(ctx), mem(mem)
     { }
-    virtual ~To_Value() { }
+    ~To_Value() { }
     using Operation<Value*>::operator();
 
     Value* operator()(Argument*);
