@@ -7,6 +7,7 @@ module SassSpec::CLI
     options = {
       engine_adapter: SassEngineAdapter.new("sass"),
       spec_directory: "spec",
+      generate: [],
       tap: false,
       skip: false,
       verbose: false,
@@ -56,6 +57,18 @@ Make sure the command you provide prints to stdout.
         options[:engine_adapter] = ExecutableEngineAdapater.new(v)
       end
 
+      opts.on("-g", "--generate format", "Run test and generate output files for the specified format or \"all\"") do |v|
+        if v == "all"
+          options[:generate].replace(options[:output_styles])
+        else
+          if options[:output_styles].include?(v)
+            options[:generate] << v
+          else
+            raise "--generate needs a valid output format #{options[:output_styles]} or \"all\""
+          end
+        end
+      end
+
       opts.on("--ignore-todo", "Skip any folder named 'todo'") do
         options[:skip_todo] = true
       end
@@ -69,7 +82,7 @@ Make sure the command you provide prints to stdout.
       end
 
       opts.on("-s", "--skip", "Skip tests that fail to exit successfully") do
-        options[:skip] = true
+        # Note: --skip is no longer necessary as this is now the default behavior, since we test for errors
       end
 
       opts.on("--nuke", "Write a new expected_output for every test from whichever engine we are using") do
