@@ -10,6 +10,22 @@ class SassSpec::Runner
     @options = options
   end
 
+  def get_files(path, arr, key)
+    entries = Dir.entries(path, {:encoding=>"UTF-8"})
+    entries.each do |entry|
+      next if entry == '.' || entry == '..'
+
+      entry_path = File.join(File.expand_path(path, Dir.pwd), entry)
+
+      if File.directory? entry_path
+        get_files(entry_path, arr, key)
+      else
+        arr << entry_path if entry == key
+      end
+    end
+    arr
+  end
+
   def run
     unless @options[:silent] || @options[:tap]
       puts "Recursively searching under directory '#{@options[:spec_directory]}' for test files to test '#{@options[:engine_adapter]}' with."
@@ -29,15 +45,14 @@ class SassSpec::Runner
       Minitest.reporter = Minitap::TapY
     end
 
-    exit Minitest.run(minioptions)
+    Minitest.run(minioptions)
   end
 
 
   def _get_cases
     cases = []
     @options[:input_files].each do |input_file|
-      glob = File.join(@options[:spec_directory], "**", input_file)
-      Dir.glob(glob) do |filename|
+      get_files(@options[:spec_directory], [], input_file).each do |filename|
         input = Pathname.new(filename)
         folder = File.dirname(filename)
         expected_stderr_file_path = File.join(folder, "error")
@@ -49,13 +64,12 @@ class SassSpec::Runner
           if ( File.file?(expected_stdout_file_path) ||
                @options[:generate].include?(output_style) ) &&
              !File.file?(expected_stdout_file_path.sub(/\.css$/, ".skip")) &&
-             filename.include?(@options[:filter]) 
+             filename.include?(@options[:filter])
             clean = File.file?(clean_file_name)
-            cases.push SassSpec::TestCase.new(input.realpath(),
+            cases.push SassSpec::TestCase.new(
+              input.realpath(),
               expected_stdout_file_path,
-              expected_stderr_file_path,
-              expected_status_file_path,
-              output_style, clean, 
+              folder, output_style, clean,
               @options[:generate].include?(output_style),
               @options)
           end
